package eu.omp.irap.streamedds; 

import java.io.IOException;
import javax.annotation.PostConstruct;
import javax.xml.bind.DatatypeConverter;
import javax.xml.bind.JAXBException;
import javax.xml.ws.WebServiceException;
import esa.egos.edds.model.JobIdPart;
import esa.egos.edds.model.PacketTCProto;
import esa.egos.edds.model.PacketTMProto;
import esa.egos.edds.notifications.NotificationChannel;
import esa.egos.edds.service.AuthenticationFault;
import esa.egos.edds.service.AuthorizationFault;
import esa.egos.edds.service.PasswordExpiredFault;
import esa.egos.edds.service.RequestFault;
import esa.egos.edds.service.SessionFault;
import esa.egos.edds.ws.client.notifications.NotificationListener;
import esa.egos.edds.ws.client.notifications.NotificationManager;
import esa.egos.edds.ws.client.EddsService;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;

public class PacketRetriever implements InitializingBean
{
	private static Logger logger = Logger.getLogger(PacketRetriever.class);
	private EddsService eddsService;
	private BodyWriter writer;
	private String username;
	private String password;
	private String serviceAddress;
	private String requestId;

	@PostConstruct
	public void init() throws WebServiceException, AuthenticationFault, PasswordExpiredFault,
			AuthorizationFault, SessionFault, RequestFault, JAXBException, IOException {
		NotificationManager manager = null;
		String sessionId = null;

		if (requestId.length() == 0) {
			return;
		}

		JobIdPart job = new JobIdPart();
		job.setJob(requestId);

		manager = new NotificationManager(eddsService, serviceAddress);
		eddsService.setNotificationManager(manager);

		eddsService.connectToServer();

		// DEBUG 2010-03-04
		PacketRetriever.logger.debug("username = '" + username + "'");
		PacketRetriever.logger.debug("service = "+ eddsService);
		PacketRetriever.logger.debug("address = "+ serviceAddress);
		PacketRetriever.logger.debug("requestid = " + requestId);
		// DEBUG 2010-03-04

		PacketRetriever.logger.info("Logging in...");
		sessionId = eddsService.logIn(username, password);

		addListeners(manager);

		eddsService.startStreamData(job);

		PacketRetriever.logger.info("Session id: " + sessionId);
		PacketRetriever.logger.info("Init complete");
	}

	protected void addListeners(NotificationManager manager) {
		manager.registerListener(NotificationChannel.PKT_TC_STREAM,
			new NotificationListener<PacketTCProto.PacketTCBinary>() {

			@Override
			public void onNotification(PacketTCProto.PacketTCBinary packet, String requestId) {
				if (PacketRetriever.this.getRequestId().equals(requestId)) {
					PacketRetriever.this.getWriter().write(packet.getBody());
				}
			}

		});

		manager.registerListener(NotificationChannel.PKT_TM_STREAM,
			new NotificationListener<PacketTMProto.PacketTMBinary>() {

			@Override
			public void onNotification(PacketTMProto.PacketTMBinary packet, String requestId) {
				if (PacketRetriever.this.getRequestId().equals(requestId)) {
					PacketRetriever.this.getWriter().write(packet.getBody());
				}
			}

		});
	}

	public EddsService getEddsService() {
		return eddsService;
	}

	public void setEddsService(EddsService eddsService) {
		this.eddsService = eddsService;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getServiceAddress() {
		return serviceAddress;
	}

	public void setServiceAddress(String serviceAddress) {
		this.serviceAddress = serviceAddress;
	}

	public BodyWriter getWriter() {
		return writer;
	}

	public void setWriter(BodyWriter writer) {
		this.writer = writer;
	}

	public String getRequestId() {
		return requestId;
	}

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

	public void afterPropertiesSet() throws Exception {
		init();
	}

}
