package eu.omp.irap.streamedds; 

import javax.annotation.PostConstruct;
import javax.xml.bind.DatatypeConverter;
import javax.xml.bind.JAXBException;
import java.io.IOException;
import esa.egos.edds.service.RequestFault;
import esa.egos.edds.service.SessionFault;
import esa.egos.edds.service.AuthorizationFault;
import esa.egos.edds.service.PasswordExpiredFault;
import esa.egos.edds.service.AuthenticationFault;
import javax.xml.ws.WebServiceException;
import esa.egos.edds.notifications.NotificationChannel;
import esa.egos.edds.ws.client.notifications.NotificationManager;
import esa.egos.edds.model.JobIdPart;
import esa.egos.edds.ws.client.EddsService;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;

public abstract class PacketRetriever implements InitializingBean
{
	private static Logger logger = Logger.getLogger(PacketRetriever.class);
	private EddsService eddsService;
	private BodyWriter writer;
	private String username;
	private String password;
	private String serviceAddress;
	private String requestId;

	@PostConstruct
	public void init() throws WebServiceException, AuthenticationFault, PasswordExpiredFault,
			AuthorizationFault, SessionFault, RequestFault, JAXBException, IOException {
		NotificationManager manager = null;
		String sessionId = null;

		if (requestId.lengh() == 0) {
			return;
		}

		JobIdPart job = new JobIdPart();
		job.setJob(requestId);

		manager = new NotificationManager(eddsService, serviceAddress);
		eddsService.setNotificationManager(manager);

		eddsService.connectToServer();

		PacketRetriever.logger.info("Logging in...");
		sessionId = eddsService.logIn(username, password);

		addListeners(manager);

		eddsService.startStreamData(job);

		PacketRetriever.logger.info("Session id: " + sessionId);
		PacketRetriever.logger.info("Init complete");
	}

	protected abstract void addListeners(NotificationManager manager);

	public EddsService getEddsService() {
		return eddsService;
	}

	public void setEddsService(EddsService eddsService) {
		this.eddsService = eddsService;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getServiceAddress() {
		return serviceAddress;
	}

	public void setServiceAddress(String serviceAddress) {
		this.serviceAddress = serviceAddress;
	}

	public BodyWriter getWriter() {
		return writer;
	}

	public void setWriter(BodyWriter writer) {
		this.writer = writer;
	}

	public String getRequestId() {
		return requestId;
	}

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

	public void afterPropertiesSet() throws Exception {
		init();
	}

}
