package eu.omp.irap.streamedds; 

import javax.annotation.PostConstruct;
import java.util.Arrays;
import java.io.PrintWriter;
import java.io.File;
import java.io.IOException;
import javax.xml.bind.DatatypeConverter;
import javax.xml.bind.JAXBException;
import esa.egos.edds.service.RequestFault;
import esa.egos.edds.service.SessionFault;
import esa.egos.edds.service.AuthorizationFault;
import esa.egos.edds.service.PasswordExpiredFault;
import esa.egos.edds.service.AuthenticationFault;
import javax.xml.ws.WebServiceException;
import esa.egos.edds.notifications.NotificationChannel;
import esa.egos.edds.streamclient.store.tm.TmToFileFactory;
import esa.egos.edds.ws.client.notifications.NotificationManager;
import esa.egos.edds.model.JobIdPart;
import esa.egos.edds.streamclient.store.StoreTmToFile;
import esa.egos.edds.ws.client.EddsService;
import org.apache.log4j.Logger;
import org.apache.activemq.protobuf.Buffer;
import org.springframework.beans.factory.InitializingBean;
import esa.egos.edds.model.PacketTMProto;
import esa.egos.edds.model.PacketTCProto;
import esa.egos.edds.ws.client.notifications.NotificationListener;

public class PacketTmStore implements InitializingBean
{
	private static Logger logger = Logger.getLogger(PacketTmStore.class);
	private String requestId;
	private EddsService eddsService;
	private String username;
	private String password;
	private String serviceAddress;
	private BodyWriter writer;
	private NotificationChannel channel;

	@PostConstruct
	public void init() throws WebServiceException, AuthenticationFault, PasswordExpiredFault,
			AuthorizationFault, SessionFault, RequestFault, JAXBException, IOException {
		NotificationManager nm = null;
		String sessionId = null;

		JobIdPart job = new JobIdPart();
		job.setJob(requestId);

		nm = new NotificationManager(eddsService, serviceAddress);
		eddsService.setNotificationManager(nm);

		eddsService.connectToServer();

		PacketTmStore.logger.info("Logging in...");
		sessionId = eddsService.logIn(username, password);

		nm.registerListener(NotificationChannel.PKT_TM_STREAM,
			new NotificationListener<PacketTMProto.PacketTMBinary>() {

			@Override
			public void onNotification(PacketTMProto.PacketTMBinary packet, String requestId) {
				if (PacketTmStore.this.requestId.equals(requestId)) {
					PacketTmStore.this.writer.write(packet.getBody());
				}
			}

		});

		eddsService.startStreamData(job);

		PacketTmStore.logger.debug("Expired: " + eddsService.hasSessionExpired());
		PacketTmStore.logger.debug("Is connected: " + nm.isConnected());
		PacketTmStore.logger.info("Session id: " + sessionId);
		PacketTmStore.logger.info("Init complete");
	}

	public EddsService getEddsService() {
		return eddsService;
	}

	public void setEddsService(EddsService eddsService) {
		this.eddsService = eddsService;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getRequestId() {
		return requestId;
	}

	public void setRequestId(String requestId) {
		this.requestId = requestId;
	}

	public String getServiceAddress() {
		return serviceAddress;
	}

	public void setServiceAddress(String serviceAddress) {
		this.serviceAddress = serviceAddress;
	}

	public BodyWriter getWriter(BodyWriter writer) {
		return writer;
	}

	public void setWriter(BodyWriter writer) {
		this.writer = writer;
	}

	public void afterPropertiesSet() throws Exception {
		init();
	}

}
